#-------------------------------------------------------------------------------
import pygame
import debug
import config
import bitmaps
import objects
import music

#-------------------------------------------------------------------------------
SWING_VEL = 6

#-------------------------------------------------------------------------------
class Hook( pygame.sprite.Sprite ):
    #---------------------------------------------------------------------------
    def __init__( self, x, y, dir, world, stringAmount ):
        self.typename = 'Hook'
        pygame.sprite.Sprite.__init__( self )

        self.stringAmount = stringAmount 

        self.begin_x = (x/32) * 32 + 16
        self.begin_y = (y/32) * 32 + 16
        if dir == 'up':
            self.init_y = (y/32) * 32 + 16
            self.init_x = (x/32) * 32
            self.real_h = self.init_y
            self.real_w = 32
            real_y = 0
            real_x = self.init_x
            self.stringBmp = bitmaps.get( 'stringV' )
            self.finalSize = self.real_h
            self.init_in = self.init_y
            self.stringDir = -1
            self.hookDir = -1
            self.vertical = True
        elif dir == 'down':
            self.init_y = (y/32) * 32 + 16
            self.init_x = (x/32) * 32
            self.real_h = config.HEIGHT - self.init_y
            self.real_w = 32
            real_y = self.init_y
            real_x = self.init_x
            self.stringBmp = bitmaps.get( 'stringV' )
            self.finalSize = self.real_h
            self.init_in = 0
            self.stringDir = 0
            self.hookDir = 1
            self.vertical = True
        elif dir == 'right':
            self.init_y = (y/32) * 32
            self.init_x = (x/32) * 32 + 16
            self.real_h = 32
            self.real_w = config.WIDTH - self.init_x
            real_y = self.init_y
            real_x = self.init_x
            self.stringBmp = bitmaps.get( 'stringH' )
            self.finalSize = self.real_w
            self.init_in = 0
            self.stringDir = 0
            self.hookDir = 1
            self.vertical = False
        elif dir == 'left':
            self.init_y = (y/32) * 32
            self.init_x = (x/32) * 32 + 16
            self.real_h = 32
            self.real_w = self.init_x - 96
            real_y = self.init_y
            real_x = 96
            self.stringBmp = bitmaps.get( 'stringH' )
            self.finalSize = self.real_w
            self.init_in = self.real_w
            self.stringDir = -1
            self.hookDir = -1
            self.vertical = False
        else:
            assert False, str( dir ) + " Incorrect hook direction"

        self.stringSize = 1
        debug.printMsg( "hook from: " + str( self.init_x ) + ", " +
                        str( self.init_y ) + "( " + str( x ) +
                        ", " + str( y ) + " )" )

        self.image = pygame.Surface((self.real_w, self.real_h)).convert()
        self.image.fill( config.COLOR_KEY )
        self.image.set_colorkey( config.COLOR_KEY )
        self.rect = self.image.get_rect().move( real_x, real_y )

        self.hookPoints = [None, None]
        self.hookPoints[0] = world.locateHookLog( self.begin_x, self.begin_y )
        self.world = world

        self.ended = False
        self.next_update_time = 0

    #---------------------------------------------------------------------------
    def isZombie( self ):
        return self.ended

    #---------------------------------------------------------------------------
    def isHookable( self ):
        return False

    #---------------------------------------------------------------------------
    def checkCollision( self, spr, rect ):
        return False

    #---------------------------------------------------------------------------
    def update( self, current ):
        if self.next_update_time < current:
            if not self.ended:
                if self.stringSize > self.finalSize:
                    self.stringSize = self.finalSize
                    self.nothingHooked()
                else:
                    self.stringSize += config.HOOK_VEL

                if self.stringSize > self.stringAmount:
                    self.nothingHooked()

                if self.vertical:
                    rectDst = pygame.Rect( 
                        0, self.init_in + self.stringDir*self.stringSize,
                        32, self.stringSize )
                    rectSrc = pygame.Rect( 0, 0, 32, self.stringSize )
                else:
                    rectDst = pygame.Rect(
                        self.init_in + self.stringDir*self.stringSize, 0,
                        self.stringSize, 32)
                    rectSrc = pygame.Rect( 0, 0, self.stringSize, 32 )
                self.image.blit( self.stringBmp, rectDst, rectDst )

                self.checkHook()

            self.next_update_time = current + config.TIME_STEP

    #---------------------------------------------------------------------------
    def setLauncher( self, launcher ):
        self.launcher = launcher

    #---------------------------------------------------------------------------
    def checkHook( self ):
        if self.hookPoints[0] and self.hookPoints[0].isUsed:
            music.playFX( music.HOOK_STRIKE )
            self.nothingHooked()
            return

        if self.vertical:
            self.hookPoints[1] = self.world.locateHookPoint( 
                self.begin_x, 
                self.begin_y + self.hookDir*self.stringSize )
        else:
            self.hookPoints[1] = self.world.locateHookPoint(
                self.begin_x + self.hookDir*self.stringSize, 
                self.begin_y )
                
        if self.hookPoints[0] == self.hookPoints[1]:
            self.hookPoints[1] = None
                
        if self.hookPoints[1]:
            if self.hookPoints[1].typename == 'HookLog':
                if self.hookPoints[0]:
                    if self.hookPoints[0].typename == 'HookLog':
                        if not self.hookPoints[1].isUsed:
                            music.playFX( music.HOOK_STRIKE )
                            self.createStringBridge()
                else:
                    music.playFX( music.HOOK_STRIKE )
                    self.nothingHooked()
            elif self.hookPoints[1].typename == 'Block':
                music.playFX( music.HOOK_STRIKE )
                self.hookBlock()
            elif self.hookPoints[1].typename == 'HookBarV':
                music.playFX( music.HOOK_STRIKE )
                self.hookBarV()
            elif self.hookPoints[1].typename == 'HookBarH':
                music.playFX( music.HOOK_STRIKE )
                self.hookBarH()
            else:
                debug.printMsg( "Unknown Hook Action" )
                self.nothingHooked()
    
    #---------------------------------------------------------------------------
    def createStringBridge( self ):
        self.ended = True
        self.world.createStringBridge( self.hookPoints[0], self.hookPoints[1] )
        point1 = self.hookPoints[0].getHookPoint().center
        point2 = self.hookPoints[1].getHookPoint().center
        dist1 = abs( point1[0] - point2[0] )
        dist2 = abs( point1[1] - point2[1] )
        self.launcher.hookEnd( max( dist1, dist2 ) )
        debug.printMsg( "Hook ended in Bridge" )

    #---------------------------------------------------------------------------
    def hookBlock( self ):
        self.ended = True
        self.launcher.hookEnd( 0 )
        self.launcher.attachBlock( self.hookPoints[1] )
        debug.printMsg( "Hook ended in Block" )

    #---------------------------------------------------------------------------
    def hookBarV( self ):
        self.ended = True
        point = self.hookPoints[1].getHookPoint().center
        self.launcher.hookEnd( abs( self.init_x - point[0] ) )
        self.launcher.attachBar()
        self.hookPoints[1].transport( self.launcher )
        music.playFX( music.SWING )
        debug.printMsg( "Hook ended in Vertical Bar" )

    #---------------------------------------------------------------------------
    def hookBarH( self ):
        self.ended = True
        point = self.hookPoints[1].getHookPoint().center
        self.launcher.hookEnd( abs( self.init_y - point[1] ) )
        self.launcher.attachBar()
        self.hookPoints[1].transport( self.launcher )
        music.playFX( music.SWING )
        debug.printMsg( "Hook ended in Horizontal Bar" )

    #---------------------------------------------------------------------------
    def nothingHooked( self ):
        self.ended = True
        self.launcher.hookEnd( 0 )
        debug.printMsg( "Hook ended with nothing" )
        
#-------------------------------------------------------------------------------
class HookLog( pygame.sprite.Sprite ):
    #---------------------------------------------------------------------------
    def __init__( self, x, y ):
        self.typename = 'HookLog'
        pygame.sprite.Sprite.__init__( self )
        self.image = pygame.Surface((32,32)).convert()
        self.image.fill( config.COLOR_KEY )
        self.rect = self.image.get_rect().move( 96 + 32*x, 32*y )

        objects = bitmaps.get( 'obj01' )
        dst = (0,0,32,32)
        src = (3*32,0,32,32)
        self.image.blit( objects, dst, src )
        self.image.set_colorkey( config.COLOR_KEY )

        self.hookPoint = pygame.Rect( 
            96 + 32*x + 16 - (config.HOOK_VEL/2), 
            32*y + 16 - (config.HOOK_VEL/2),
            config.HOOK_VEL, config.HOOK_VEL )

        self.isUsed = False

    #---------------------------------------------------------------------------
    def isZombie( self ):
        return False

    #---------------------------------------------------------------------------
    def isHookable( self ):
        return True

    #---------------------------------------------------------------------------
    def checkCollision( self, spr, rect ):
        return False

    #---------------------------------------------------------------------------
    def getHookPoint( self ):
        return self.hookPoint


#-------------------------------------------------------------------------------
class HookBarV( pygame.sprite.Sprite ):
    #---------------------------------------------------------------------------
    def __init__( self, x, y, world ):
        self.typename = 'HookBarV'
        pygame.sprite.Sprite.__init__( self )
        self.image = pygame.Surface((32,32)).convert()
        self.image.fill( config.COLOR_KEY )
        self.rect = self.image.get_rect().move( 96 + 32*x, 32*y )

        objects = bitmaps.get( 'obj01' )
        dst = (0,0,32,32)
        src = (6*32,0,32,32)
        self.image.blit( objects, dst, src )
        self.image.set_colorkey( config.COLOR_KEY )

        self.hookPoint = pygame.Rect( 
            96 + 32*x + 14, 32*y + 14, 4, 4 )

        self.next_update_time = 0
        self.hookSprite = None
        self.world = world

    #---------------------------------------------------------------------------
    def isZombie( self ):
        return False

    #---------------------------------------------------------------------------
    def isHookable( self ):
        return True

    #---------------------------------------------------------------------------
    def checkCollision( self, spr, rect ):
        return False

    #---------------------------------------------------------------------------
    def update( self, current ):
        if self.next_update_time < current:
            if self.hookSprite:
                self.current_x += self.inc * SWING_VEL
                self.hookSprite.rect.x = self.current_x
                self.string.setSize( 
                    self.current_x + 16, self.hookPoint.center[0], 
                    self.hookPoint.center[1] - 16 )
                if self.arriveFun( self.current_x, self.final_x ):
                    self.hookSprite.rect.x = self.final_x
                    self.hookSprite.swingEnd()
                    self.hookSprite = None
                    self.string.destroy()
                    self.string = None
            self.next_update_time = current + config.TIME_STEP
        
    #---------------------------------------------------------------------------
    def getHookPoint( self ):
        return self.hookPoint

    #---------------------------------------------------------------------------
    def transport( self, sprite ):
        self.hookSprite = sprite

        init_x = self.hookSprite.rect.x
        init_y = self.hookSprite.rect.y

        end_x = self.hookPoint.center[0]
        end_y = self.hookPoint.center[1]

        dist_x = abs(init_x - end_x)
        dist_y = abs(init_y - end_y)
        if dist_x < dist_y:
            self.hookSprite.swingEnd()
            return

        self.inc = (end_x - init_x)/dist_x
        self.current_x = init_x
        self.final_x = init_x + 2*(end_x - (init_x + 32)) + 32
        self.final_y = init_y
        if self.inc > 0:
            self.arriveFun = lambda x0, x1: x0 >= x1
        else:
            self.arriveFun = lambda x0, x1: x0 <= x1

        self.string = objects.SwingStringV()
        self.world.insertSprite( self.string )

#-------------------------------------------------------------------------------
class HookBarH( pygame.sprite.Sprite ):
    #---------------------------------------------------------------------------
    def __init__( self, x, y, world ):
        self.typename = 'HookBarH'
        pygame.sprite.Sprite.__init__( self )
        self.image = pygame.Surface((32,32)).convert()
        self.image.fill( config.COLOR_KEY )
        self.rect = self.image.get_rect().move( 96 + 32*x, 32*y )

        objects = bitmaps.get( 'obj01' )
        dst = (0,0,32,32)
        src = (7*32,0,32,32)
        self.image.blit( objects, dst, src )
        self.image.set_colorkey( config.COLOR_KEY )

        self.hookPoint = pygame.Rect( 
            96 + 32*x + 14, 32*y + 14, 4, 4 )

        self.next_update_time = 0
        self.hookSprite = None
        self.world = world

    #---------------------------------------------------------------------------
    def isZombie( self ):
        return False

    #---------------------------------------------------------------------------
    def isHookable( self ):
        return True

    #---------------------------------------------------------------------------
    def checkCollision( self, spr, rect ):
        return False

    #---------------------------------------------------------------------------
    def update( self, current ):
        if self.next_update_time < current:
            if self.hookSprite:
                self.current_y += self.inc * SWING_VEL
                self.hookSprite.rect.y = self.current_y
                self.string.setSize( 
                    self.current_y + 16, self.hookPoint.center[1], 
                    self.hookPoint.center[0] - 16 )
                if self.arriveFun( self.current_y, self.final_y ):
                    self.hookSprite.rect.y = self.final_y
                    self.hookSprite.swingEnd()
                    self.hookSprite = None
                    self.string.destroy()
                    self.string = None
            self.next_update_time = current + config.TIME_STEP
        
    #---------------------------------------------------------------------------
    def getHookPoint( self ):
        return self.hookPoint

    #---------------------------------------------------------------------------
    def transport( self, sprite ):
        self.hookSprite = sprite

        init_x = self.hookSprite.rect.x
        init_y = self.hookSprite.rect.y

        end_x = self.hookPoint.center[0]
        end_y = self.hookPoint.center[1]

        dist_x = abs(init_x - end_x)
        dist_y = abs(init_y - end_y)
        if dist_x > dist_y:
            self.hookSprite.swingEnd()
            return

        self.inc = (end_y - init_y)/dist_y
        self.current_y = init_y
        self.final_y = init_y + 2*(end_y - init_y) - 50
        self.final_x = init_x
        if self.inc > 0:
            self.arriveFun = lambda y0, y1: y0 >= y1
        else:
            self.arriveFun = lambda y0, y1: y0 <= y1

        self.string = objects.SwingStringH()
        self.world.insertSprite( self.string )

#-------------------------------------------------------------------------------
