#-------------------------------------------------------------------------------
import data
import pygame
import debug
import music
import hook
import objects

#-------------------------------------------------------------------------------
class Level:

    #---------------------------------------------------------------------------
    def __init__( self, world, idx ):
        self.world = world
        self.lvlDef = m_levels[ idx ]
        self.lvlData = pygame.image.load( data.filepath( self.lvlDef[0] ) )
        self.lvlTiles = pygame.image.load( data.filepath( self.lvlDef[1] ) )
        self.playerInit = (0,0)
        music.play( self.lvlDef[2] )
        debug.printMsg( "Created Level #" + str( idx ) )

        self.dynTiles = range(0,15)
        for j in range(0,15):
            self.dynTiles[j] = range(0,17)
            for i in range(0,17):
                self.dynTiles[j][i] = 0

    #---------------------------------------------------------------------------
    def draw( self, bg ):
        levelBG = pygame.image.load( 
            data.filepath( self.lvlDef[3] ) ).convert_alpha()
        rect = levelBG.get_rect()
        rect.x = 32 * 3
        bg.blit( levelBG, rect )

        for j in range(0,15):
            for i in range(0,17):
                tileData = self.lvlData.get_at( (i,j) )
                if tileData[0] & 128:
                    tile = tileData[0] - 128
                    dst = (i*32 + 96, j*32, 32, 32 )
                    src = ((tile%10)*32, (tile/10)*32, 32, 32)
                    bg.blit( self.lvlTiles, dst, src )
                    
                if tileData[1] & 128:
                    obj = tileData[1] - 128
                    
                    if obj < len( m_createFuns ):
                        m_createFuns[obj]( 
                            self, i, j, tileData[2], tileData[3] )
                    else:
                        debug.printMsg( "Unknown level obj: " + str( obj ) )

    #---------------------------------------------------------------------------
    def createPlayerInit( self, i, j, param1, param2 ):
        self.playerInit = (i,j)
    
    #---------------------------------------------------------------------------
    def createExitDoor( self, i, j, param1, param2 ):
        myExitDoor = objects.ExitDoor( i, j )
        self.world.insertObjSprite( myExitDoor )

    #---------------------------------------------------------------------------
    def createHookLog( self, i, j, param1, param2 ):
        myHookLog = hook.HookLog( i, j )
        self.world.insertObjSprite( myHookLog )

    #---------------------------------------------------------------------------
    def createBlock( self, i, j, param1, param2 ):
        myBlock = objects.Block( i, j )
        self.world.insertObjSprite( myBlock, False )

    #---------------------------------------------------------------------------
    def createHookBarV( self, i, j, param1, param2 ):
        myHookBar = hook.HookBarV( i, j, self.world )
        self.world.insertObjSprite( myHookBar, False )

    #---------------------------------------------------------------------------
    def createHookBarH( self, i, j, param1, param2 ):
        myHookBar = hook.HookBarH( i, j, self.world )
        self.world.insertObjSprite( myHookBar, False )

    #---------------------------------------------------------------------------
    def getPlayerBegin( self ):
        return self.playerInit

    #---------------------------------------------------------------------------
    def isOnTile( self, x, y ):
        #return True
        pos = ( x/32,  y/32 )
        data = self.lvlData.get_at( pos )
        if (data[0] & 128) == 128:
            return True

        return (self.dynTiles[ pos[1] ][ pos[0] ] > 0)

    #---------------------------------------------------------------------------
    def isOnTile2( self, x, y ):
        pos = ( x,  y )
        data = self.lvlData.get_at( pos )
        if (data[0] & 128) == 128:
            return True

        return (self.dynTiles[ pos[1] ][ pos[0] ] > 0)

    #---------------------------------------------------------------------------
    def isOnTileSize( self, x, y, size ):
        x0 = (x - size)/32
        x1 = (x + size)/32
        y1 = (y + size)/32
        while x0 <= x1:
            y0 = (y - size)/32
            while y0 <= y1:
                if self.isOnTile2( x0, y0 ):
                    return True
                y0 += 1
            x0 += 1
        return False
            

    #---------------------------------------------------------------------------
    def addBridge( self, point1, point2 ):
        p1_x = (point1[0] - 96)/32
        p1_y = point1[1]/32
        p2_x = (point2[0] - 96)/32
        p2_y = point2[1]/32

        tiles = []
        j = p1_y
        while j <= p2_y:
            i = p1_x
            while i <= p2_x:
                tiles.append( (i,j) )
                self.dynTiles[ j ][ i ] += 1
                i += 1
            j += 1

        return tiles

    #---------------------------------------------------------------------------
    def removeTiles( self, tiles ):
        for t in tiles:
            self.dynTiles[ t[1] ][ t[0] ] -= 1

    #---------------------------------------------------------------------------
    def getDefaultStringAmount( self ):
        return self.lvlDef[ 4 ]

#---------------------------------------------------------------------------
def maxLevels():
    return len( m_levels )
#-------------------------------------------------------------------------------
m_levels = [
    ( "level/level01.png", "gfx/tiles01.png",
      "mus/music01.ogg", "gfx/bg01.png",
      0 ),
    ( "level/level02.png", "gfx/tiles01.png", 
      "mus/music02.ogg", "gfx/bg01.png",
      10 ),
    ( "level/level03.png", "gfx/tiles01.png", 
      "mus/music01.ogg", "gfx/bg01.png",
      5 ),
    ( "level/level04.png", "gfx/tiles01.png", 
      "mus/music02.ogg", "gfx/bg01.png",
      11 ),
    ( "level/level05.png", "gfx/tiles01.png", 
      "mus/music01.ogg", "gfx/bg01.png",
      15 )
    ]

#-------------------------------------------------------------------------------
m_createFuns = [ 
    Level.createPlayerInit ,
    Level.createExitDoor,
    Level.createHookLog,
    Level.createBlock,
    Level.createHookBarV,
    Level.createHookBarH
    ]

#-------------------------------------------------------------------------------
