#-------------------------------------------------------------------------------
import pygame
import config
import data
import debug
import config
import bitmaps
import objects
import hook
import music

#-------------------------------------------------------------------------------
class Player( pygame.sprite.Sprite ):
    NUM_FRAMES = 10
    PLAYER_VEL = 2

    ST_STOP = 0
    ST_MOVING = 1
    ST_SHOOTING = 2
    ST_DEAD = 3
    ST_FULLD = 4
    ST_BLOCKATTACH = 5
    ST_BARATTACH = 6

    DirsNames = [ 'up', 'right', 'down', 'left' ]
    DirsMap = { 'up':0, 'right':1, 'down':2, 'left':3, 'dead':4 }
    DirsInc = [ (0,-PLAYER_VEL), (PLAYER_VEL,0), 
                (0,PLAYER_VEL), (-PLAYER_VEL,0) ]
    #---------------------------------------------------------------------------
    def __init__( self, position ):
        self.typename = 'Player'
        pygame.sprite.Sprite.__init__( self )
        self.imageList = bitmaps.get( 'player' )

        self.image = pygame.Surface((32,32)).convert( self.imageList )
        self.image.fill( config.COLOR_KEY )
        self.image.blit( self.imageList, self.image.get_rect() )
        self.image.set_colorkey( config.COLOR_KEY )
        self.rect = self.image.get_rect().move( position )
        self.dir = Player.DirsMap['down']
        self.currentAnim = 0
        self.newAnim = self.currentAnim
        self.currentFrame = 0
        self.next_update_time = 0
        self.state = Player.ST_STOP
        self.string = False
        self.stringAmount = 0

        self.stateCheck = debug.Variable( 
            config.WIDTH - 100, debug.nexty(), 'player state', True )
        
    #---------------------------------------------------------------------------
    def setStringAmount( self, size ):
        self.stringAmount = size * 32

    #---------------------------------------------------------------------------
    def getStringAmount( self ):
        return self.stringAmount / 32

    #---------------------------------------------------------------------------
    def isHookable( self ):
        return False

    #---------------------------------------------------------------------------
    def checkCollision( self, spr, rect ):
        return False

    #---------------------------------------------------------------------------
    def isZombie( self ):
        return False

    #---------------------------------------------------------------------------
    def setLevel( self, lvl ):
        self.level = lvl

    #---------------------------------------------------------------------------
    def setWorld( self, world ):
        self.world = world

    #---------------------------------------------------------------------------
    def isDead( self ):
        return (self.state == Player.ST_DEAD) or (self.state == Player.ST_FULLD)
    
    #---------------------------------------------------------------------------
    def isFullDead( self ):
        return (self.state == Player.ST_FULLD)

    #---------------------------------------------------------------------------
    def isMoving( self ):
        return (self.state == Player.ST_MOVING)

    #---------------------------------------------------------------------------
    def isShooting( self ):
        return (self.state == Player.ST_SHOOTING)

    #---------------------------------------------------------------------------
    def isAttached( self ):
        return ((self.state == Player.ST_BLOCKATTACH) or
                (self.state == Player.ST_BARATTACH) )

    #---------------------------------------------------------------------------
    def isSwinging( self ):
        return (self.state == Player.ST_BARATTACH)

    #---------------------------------------------------------------------------
    def canShot( self ):
        if self.stringAmount < 32:
            return False
        if self.isDead():
            return False
        return ((self.state != Player.ST_SHOOTING) and 
                (self.state != Player.ST_BARATTACH) )

    #---------------------------------------------------------------------------
    def canStop( self ):
        if self.isDead():
            return False
        return not self.isShooting() and not self.isAttached()

    #---------------------------------------------------------------------------
    def canMove( self ):
        if self.isDead():
            return False
        return not self.isShooting() and not self.isAttached()

    #---------------------------------------------------------------------------
    def update( self, current ):
        if self.next_update_time < current:
            if self.state == Player.ST_BLOCKATTACH:
                self.rect = self.rect.move( self.blockInc )
                self.string.move( self.blockInc )
                self.block.move( self.blockInc )
                bck_point = self.block.getHookPoint().center
                levelx = bck_point[0] - 96
                levely = bck_point[1]
                if not self.level.isOnTileSize( levelx, levely, 10 ):
                    music.playFX( music.BOX_DROP )
                    self.block.destroy()
                    self.block = None
                    self.string.destroy()
                    self.string = None
                    distx = abs( (self.rect.x + 16) - bck_point[0])
                    disty = abs( (self.rect.y + 16) - bck_point[1])
                    self.hookEnd( max(distx, disty) )
            if self.isMoving():
                newRect = self.rect.move( Player.DirsInc[self.dir] )
                if not self.world.checkCollision( self, newRect.inflate(-12,-1) ):
                    self.rect = newRect
                    if self.rect.x < 96:
                        self.rect.x = 96
                    if self.rect.x > config.WIDTH - 32:
                        self.rect.x = config.WIDTH - 32
                    if self.rect.y < 0:
                        self.rect.y = 0
                    if self.rect.y > config.HEIGHT - 32:
                        self.rect.y = config.HEIGHT - 32
            if not self.isDead() and not self.isSwinging():
                levelx = self.rect.x - 96 + 16
                levely = self.rect.y + 24
                if not self.level.isOnTile( levelx, levely ):
                    music.playFX( music.PLAYER_DIE )
                    self.state = Player.ST_DEAD
                    self.dir = Player.DirsMap[ 'dead' ]
                    self.currentFrame = 0
                    self.newAnim = 0
                    self.currentAnim = 1
                    if self.string:
                        self.string.destroy()
                        self.string = None
            self.updateAnim()
            self.updateSprite()

            self.stateCheck.setValue( self.state )
            self.next_update_time = current + config.TIME_STEP

    #---------------------------------------------------------------------------
    def updateSprite( self ):
        if self.newAnim != self.currentAnim:
            self.image.fill( config.COLOR_KEY )
            dest = self.image.get_rect()
            src = dest.move( 32 * self.newAnim, 32 * self.dir )
            self.image.blit( self.imageList, dest, src )
            self.currentAnim = self.newAnim

    #---------------------------------------------------------------------------
    def updateAnim( self ):
        if self.isMoving():
            self.currentFrame += 1
            if self.currentFrame > Player.NUM_FRAMES:
                self.currentFrame = 0
                self.newAnim = (self.currentAnim + 1) % 2
        if self.isDead():
            self.currentFrame += 1
            if self.currentFrame > Player.NUM_FRAMES:
                self.currentFrame = 0
                self.newAnim = self.currentAnim + 1
                if self.newAnim > 2:
                    self.image.set_alpha( 0 )
                    self.state = Player.ST_FULLD

    #---------------------------------------------------------------------------
    def moveTo( self, newDir ):
        if self.state == Player.ST_BLOCKATTACH:
            if newDir == self.blockDir:
                self.blockInc = Player.DirsInc[ 
                    Player.DirsMap[ self.blockDir ] ]
        if self.canMove():
            if not self.isMoving() or (Player.DirsMap[ newDir ] != self.dir):
                self.dir = Player.DirsMap[ newDir ]
                self.currentFrame = 0
                self.newAnim = 1
                self.currentAnim = 0
            self.state = Player.ST_MOVING
            
    #---------------------------------------------------------------------------
    def stop( self ):
        if self.state == Player.ST_BLOCKATTACH:
            self.blockInc = (0,0)
        if self.canStop():
            self.state = Player.ST_STOP
            self.currentFrame = 0
            self.newAnim = 0
            self.currentAnim = 1

    #---------------------------------------------------------------------------
    def hookShot( self ):
        if self.canShot():
            if self.state == Player.ST_BLOCKATTACH:
                self.string.destroy()
                self.string = None
                bck_point = self.block.getHookPoint().center
                distx = abs( (self.rect.x + 16) - bck_point[0])
                disty = abs( (self.rect.y + 16) - bck_point[1])
                self.hookEnd( max(distx, disty) )
            else:
                self.currentFrame = 0
                self.newAnim = 2
                self.currentAnim = 1
                self.state = Player.ST_SHOOTING
            
                launchx = self.rect.x + 16
                launchy = self.rect.y + 16
                myHook = hook.Hook( 
                    launchx, launchy, 
                    Player.DirsNames[ self.dir ],
                    self.world, self.stringAmount )
                myHook.setLauncher( self )
                self.world.insertSprite( myHook )
                music.playFX( music.HOOK_LAUNCH )

    #---------------------------------------------------------------------------
    def hookEnd( self, amount ):
        assert amount <= self.stringAmount, "Too much string used"

        self.state = Player.ST_STOP
        self.currentFrame = 0
        self.newAnim = 0
        self.currentAnim = 1
        self.stringAmount = max( 0, self.stringAmount - amount )

    #---------------------------------------------------------------------------
    def swingEnd( self ):
        self.state = Player.ST_STOP
        self.currentFrame = 0
        self.newAnim = 0
        self.currentAnim = 1

    #---------------------------------------------------------------------------
    def spawn( self, pos ):
        self.image = pygame.Surface((32,32)).convert( self.imageList )
        self.image.fill( config.COLOR_KEY )
        self.image.blit( self.imageList, self.image.get_rect() )
        self.image.set_colorkey( config.COLOR_KEY )
        self.rect = self.image.get_rect().move( pos )
        self.dir = Player.DirsMap['down']
        self.currentAnim = 1
        self.newAnim = 0
        self.state = Player.ST_STOP

    #---------------------------------------------------------------------------
    def attachBlock( self, block ):
        plr_x = ((self.rect.x + 16 - 96)/32) * 32 + 96
        plr_y = ((self.rect.y + 16)/32) * 32
        bck_point = block.getHookPoint().center
        bck_x = ((bck_point[0] - 96)/32) * 32 + 96
        bck_y = (bck_point[1]/32) * 32
        self.string = objects.String( (plr_x, plr_y), (bck_x, bck_y) )
        self.world.insertSprite( self.string )
        
        if abs(plr_x - bck_x) > abs(plr_y - bck_y):
            if plr_x < bck_x:
                self.blockDir = 'left'
            else:
                self.blockDir = 'right'
        else:
            if plr_y < bck_y:
                self.blockDir = 'up'
            else:
                self.blockDir = 'down'
        self.blockInc = (0,0)
        self.block = block
        
        self.state = Player.ST_BLOCKATTACH

    #---------------------------------------------------------------------------
    def attachBar( self ):
        self.state = Player.ST_BARATTACH

    #---------------------------------------------------------------------------
    def exitLevel( self ):
        self.world.exitLevel()

#-------------------------------------------------------------------------------
